@@ -45,13 +45,9 @@
 
 G_DEFINE_TYPE (ScreenshotApplication, screenshot_application, GTK_TYPE_APPLICATION);
 
-static ScreenshotApplication *_app_singleton = NULL;
-
 static void screenshot_save_to_file (ScreenshotApplication *self);
 
 struct _ScreenshotApplicationPriv {
-  GDBusConnection *connection;
-
   gchar *icc_profile_base64;
   GdkPixbuf *screenshot;
 
@@ -119,7 +115,7 @@ save_pixbuf_handle_success (ScreenshotApplication *self)
       ScreenshotDialog *dialog = self->priv->dialog;
 
       save_folder_to_settings (self);
-      gtk_widget_destroy (screenshot_dialog_get_toplevel (dialog));
+      gtk_widget_destroy (dialog->window);
     }
   else
     {
@@ -134,7 +130,6 @@ save_pixbuf_handle_error (ScreenshotApplication *self,
   if (screenshot_config->interactive)
     {
       ScreenshotDialog *dialog = self->priv->dialog;
-      GtkWidget *toplevel = screenshot_dialog_get_toplevel (dialog);
 
       screenshot_dialog_set_busy (dialog, FALSE);
 
@@ -148,7 +143,7 @@ save_pixbuf_handle_error (ScreenshotApplication *self,
                                            file_name, folder_name);
           gint response;
                                              
-          response = screenshot_show_dialog (GTK_WINDOW (toplevel),
+          response = screenshot_show_dialog (GTK_WINDOW (dialog->window),
                                              GTK_MESSAGE_WARNING,
                                              GTK_BUTTONS_YES_NO,
                                              _("Overwrite existing file?"),
@@ -169,19 +164,22 @@ save_pixbuf_handle_error (ScreenshotApplication *self,
         }
       else
         {
-          screenshot_show_dialog (GTK_WINDOW (toplevel),
+          screenshot_show_dialog (GTK_WINDOW (dialog->window),
                                   GTK_MESSAGE_ERROR,
                                   GTK_BUTTONS_OK,
                                   _("Unable to capture a screenshot"),
                                   _("Error creating file. Please choose another location and retry."));
         }
 
-      screenshot_dialog_focus_entry (dialog);
+      gtk_widget_grab_focus (dialog->filename_entry);
     }
   else
     {
+      g_critical ("Unable to save the screenshot: %s", error->message);
       screenshot_play_sound_effect ("dialog-error", _("Unable to capture a screenshot"));
       g_application_release (G_APPLICATION (self));
+      if (screenshot_config->file != NULL)
+        exit (EXIT_FAILURE);
     }
 }
 
@@ -319,7 +317,6 @@ build_filename_ready_cb (GObject *source,
                          gpointer user_data)
 {
   ScreenshotApplication *self = user_data;
-  GtkWidget *toplevel;
   GError *error = NULL;
 
   self->priv->save_uri = screenshot_build_filename_finish (res, &error);
@@ -340,7 +337,11 @@ build_filename_ready_cb (GObject *source,
                                 _("Unable to capture a screenshot"),
                                 _("Error creating file"));
       else
-        screenshot_play_sound_effect ("dialog-error", _("Unable to capture a screenshot"));
+        {
+          screenshot_play_sound_effect ("dialog-error", _("Unable to capture a screenshot"));
+          if (screenshot_config->file != NULL)
+            exit (EXIT_FAILURE);
+        }
 
       return;
     }
@@ -350,12 +351,7 @@ build_filename_ready_cb (GObject *source,
   if (screenshot_config->interactive)
     {
       self->priv->dialog = screenshot_dialog_new (self->priv->screenshot, self->priv->save_uri);
-      toplevel = screenshot_dialog_get_toplevel (self->priv->dialog);
-      gtk_widget_show (toplevel);
-
-      gtk_application_add_window (GTK_APPLICATION (self), GTK_WINDOW (toplevel));
-  
-      g_signal_connect (toplevel,
+      g_signal_connect (self->priv->dialog->dialog,
                         "response",
                         G_CALLBACK (screenshot_dialog_response_cb),
                         self);
@@ -389,6 +385,8 @@ finish_prepare_screenshot (ScreenshotApplication *self,
         screenshot_play_sound_effect ("dialog-error", _("Unable to capture a screenshot"));
 
       g_application_release (G_APPLICATION (self));
+      if (screenshot_config->file != NULL)
+        exit (EXIT_FAILURE);
 
       return;
     }
@@ -428,7 +426,14 @@ finish_prepare_screenshot (ScreenshotApplication *self,
    *
    * screenshot_ensure_icc_profile (window);
    */
-  screenshot_build_filename_async (build_filename_ready_cb, self);
+  if (screenshot_config->file != NULL)
+    {
+      self->priv->save_uri = g_file_get_uri (screenshot_config->file);
+      self->priv->should_overwrite = TRUE;
+      screenshot_save_to_file (self);
+    }
+  else
+    screenshot_build_filename_async (build_filename_ready_cb, self);
 }
 
 static void
@@ -495,18 +500,22 @@ screenshot_application_local_command_line (GApplication *app,
   gboolean area_arg = FALSE;
   gboolean include_border_arg = FALSE;
   gboolean disable_border_arg = FALSE;
+  gboolean include_pointer_arg = FALSE;
   gboolean interactive_arg = FALSE;
   gchar *border_effect_arg = NULL;
   guint delay_arg = 0;
+  gchar *file_arg = NULL;
   const GOptionEntry entries[] = {
     { "clipboard", 'c', 0, G_OPTION_ARG_NONE, &clipboard_arg, N_("Send the grab directly to the clipboard"), NULL },
     { "window", 'w', 0, G_OPTION_ARG_NONE, &window_arg, N_("Grab a window instead of the entire screen"), NULL },
     { "area", 'a', 0, G_OPTION_ARG_NONE, &area_arg, N_("Grab an area of the screen instead of the entire screen"), NULL },
     { "include-border", 'b', 0, G_OPTION_ARG_NONE, &include_border_arg, N_("Include the window border with the screenshot"), NULL },
     { "remove-border", 'B', 0, G_OPTION_ARG_NONE, &disable_border_arg, N_("Remove the window border from the screenshot"), NULL },
+    { "include-pointer", 'p', 0, G_OPTION_ARG_NONE, &include_pointer_arg, N_("Include the pointer with the screenshot"), NULL },
     { "delay", 'd', 0, G_OPTION_ARG_INT, &delay_arg, N_("Take screenshot after specified delay [in seconds]"), N_("seconds") },
     { "border-effect", 'e', 0, G_OPTION_ARG_STRING, &border_effect_arg, N_("Effect to add to the border (shadow, border or none)"), N_("effect") },
     { "interactive", 'i', 0, G_OPTION_ARG_NONE, &interactive_arg, N_("Interactively set options"), NULL },
+    { "file", 'f', 0, G_OPTION_ARG_FILENAME, &file_arg, N_("Save screenshot directly to this file"), N_("filename") },
     { NULL },
   };
 
@@ -538,9 +547,11 @@ screenshot_application_local_command_line (GApplication *app,
                                 area_arg,
                                 include_border_arg,
                                 disable_border_arg,
+                                include_pointer_arg,
                                 border_effect_arg,
                                 delay_arg,
-                                interactive_arg);
+                                interactive_arg,
+                                file_arg);
 
   if (!res)
     {
@@ -558,6 +569,8 @@ screenshot_application_local_command_line (GApplication *app,
 
  out:
   g_option_context_free (context);
+  g_free (border_effect_arg);
+  g_free (file_arg);
 
   return TRUE;	
 }
@@ -598,63 +611,103 @@ interactive_dialog_response_cb (GtkWidget *d,
 {
   ScreenshotApplication *self = user_data;
 
-  gtk_widget_destroy (d);
+  if (response != GTK_RESPONSE_HELP)
+    gtk_widget_destroy (d);
 
   switch (response)
     {
     case GTK_RESPONSE_DELETE_EVENT:
-    case GTK_RESPONSE_CANCEL:
       break;
     case GTK_RESPONSE_OK:
       screenshot_start (self);
       break;
+    case GTK_RESPONSE_HELP:
+      screenshot_display_help (GTK_WINDOW (d));
+      break;
     default:
       g_assert_not_reached ();
       break;
     }
 }
 
-static ScreenshotApplication *
-get_singleton (void)
+static void
+action_quit (GSimpleAction *action,
+             GVariant *parameter,
+             gpointer user_data)
 {
-  if (_app_singleton == NULL)
-    _app_singleton = g_object_new (SCREENSHOT_TYPE_APPLICATION, 
-                                   "application-id", "org.gnome.Screenshot",
-                                   NULL);
-
-  return _app_singleton;
+  GList *windows = gtk_application_get_windows (GTK_APPLICATION (user_data));
+  gtk_widget_destroy (g_list_nth_data (windows, 0));
 }
 
+static void
+action_help (GSimpleAction *action,
+             GVariant *parameter,
+             gpointer user_data)
+{
+  GList *windows = gtk_application_get_windows (GTK_APPLICATION (user_data));
+  screenshot_display_help (g_list_nth_data (windows, 0));
+}
+
+static void
+action_about (GSimpleAction *action,
+              GVariant *parameter,
+              gpointer user_data)
+{
+  const gchar *authors[] = {
+    "Emmanuele Bassi",
+    "Jonathan Blandford",
+    "Cosimo Cecchi",
+    NULL
+  };
+
+  GList *windows = gtk_application_get_windows (GTK_APPLICATION (user_data));
+  gtk_show_about_dialog (GTK_WINDOW (g_list_nth_data (windows, 0)),
+                         "version", VERSION,
+                         "authors", authors,
+                         "program-name", _("Screenshot"),
+                         "comments", _("Save images of your screen or individual windows"),
+                         "logo-icon-name", "applets-screenshooter",
+                         "translator-credits", _("translator-credits"),
+                         "license-type", GTK_LICENSE_GPL_2_0,
+                         "wrap-license", TRUE,
+                         NULL);
+}
+
+static GActionEntry action_entries[] = {
+  { "about", action_about, NULL, NULL, NULL },
+  { "help", action_help, NULL, NULL, NULL },
+  { "quit", action_quit, NULL, NULL, NULL }
+};
+
 static void
 screenshot_application_startup (GApplication *app)
 {
   ScreenshotApplication *self = SCREENSHOT_APPLICATION (app);
-  GError *error = NULL;
 
   G_APPLICATION_CLASS (screenshot_application_parent_class)->startup (app);
 
   gtk_window_set_default_icon_name (SCREENSHOOTER_ICON);
   screenshooter_init_stock_icons ();
 
-  self->priv->connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
-
-  if (error != NULL)
-    {
-      g_critical ("Unable to connect to the session bus: %s",
-                  error->message);
-      g_error_free (error);
-
-      return;
-    }
-
   /* interactive mode: trigger the dialog and wait for the response */
   if (screenshot_config->interactive)
     {
       GtkWidget *dialog;
+      GtkBuilder *builder;
+      GMenuModel *menu;
+
+      g_action_map_add_action_entries (G_ACTION_MAP (self), action_entries,
+                                       G_N_ELEMENTS (action_entries), self);
+
+      builder = gtk_builder_new ();
+      gtk_builder_add_from_resource (builder, "/org/gnome/screenshot/screenshot-app-menu.ui", NULL);
+      menu = G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu"));
+      gtk_application_set_app_menu (GTK_APPLICATION (self), menu);
+
+      g_object_unref (builder);
+      g_object_unref (menu);
 
       dialog = screenshot_interactive_dialog_new ();
-      gtk_application_add_window (GTK_APPLICATION (app), GTK_WINDOW (dialog));
-      gtk_widget_show (dialog);
       g_signal_connect (dialog, "response",
                         G_CALLBACK (interactive_dialog_response_cb), self);
     }
@@ -669,7 +722,6 @@ screenshot_application_finalize (GObject *object)
 {
   ScreenshotApplication *self = SCREENSHOT_APPLICATION (object);
 
-  g_clear_object (&self->priv->connection);
   g_clear_object (&self->priv->screenshot);
   g_free (self->priv->icc_profile_base64);
   g_free (self->priv->save_uri);
@@ -698,15 +750,10 @@ screenshot_application_init (ScreenshotApplication *self)
                                             ScreenshotApplicationPriv);
 }
 
-GDBusConnection *
-screenshot_application_get_session_bus (void)
-{
-  ScreenshotApplication *self = get_singleton ();
-  return self->priv->connection;
-}
-
 ScreenshotApplication *
-screenshot_application_get (void)
+screenshot_application_new (void)
 {
-  return get_singleton ();
+  return g_object_new (SCREENSHOT_TYPE_APPLICATION, 
+                       "application-id", "org.gnome.Screenshot",
+                       NULL);
 }
